/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 2.0
 */

using System;
using System.Runtime.Serialization;

namespace TopCoder.Web.UI.WebControl.BreadCrumb
{
    /// <summary>
    /// Exception thrown in all cases where the <c>BreadCrumbTrailControl</c> cannot correctly parse, format and write
    /// bread crumb trails.
    /// </summary>
    /// <remarks>
    /// How these exceptions are actually handled depends, among other things, on the <see cref="ErrorDisposition"/>
    /// setting.
    /// </remarks>
    [Serializable]
    public class BreadCrumbException : ApplicationException
    {
        /// <summary>
        /// Constructs the exception with no information.
        /// </summary>
        public BreadCrumbException()
            : base()
        {
        }

        /// <summary>
        /// Constructs the exception from the given message.
        /// </summary>
        /// <param name="message">A message string, possibly null, possibly empty.</param>
        public BreadCrumbException(string message)
            : base(message)
        {
        }

        /// <summary>
        /// Constructs the exception from the given message and inner exception.
        /// </summary>
        /// <param name="message">A message string, possibly null, possibly empty.</param>
        /// <param name="cause">The inner exception/cause, possibly null.</param>
        public BreadCrumbException(string message, Exception cause)
            : base(message, cause)
        {
        }

        /// <summary>
        /// Constructs the exception from the given serialization information, used in deserialization.
        /// </summary>
        /// <param name="info">The deserialization info.</param>
        /// <param name="context">The streaming context (not actually used in .NET 1.1).</param>
        protected BreadCrumbException(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
        }
    }
}
