/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 2.0
 */

using System;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery
{
    /// <summary>
    /// Defines the contract for a node matcher.
    /// </summary>
    /// <remarks>
    /// <para>Implementations need to provide three functions:</para>
    /// <list>
    /// <item>A matching function that will take a node and determine if it matches.</item>
    /// <item>A default title string used for those nodes that match.</item>
    /// <item>A default url to use if no contextual information is available.</item>
    /// </list>
    /// <para>
    /// Implementations of this interface MUST override the equals/hash code methods because the implementation will be
    /// stored in a (hash) dictionary. Implementations of this interface should be thread safe.
    /// </para>
    /// </remarks>
    public interface INodeMatcher
    {
        /// <summary>
        /// Returns the default title for any node that matches this node matcher. The default title can be used if the
        /// node has no title assigned to it.
        /// </summary>
        /// <value>
        /// A title string, not null, not empty.
        /// </value>
        string Title
        {
            get;
        }

        /// <summary>
        /// Returns the default url that can be used if contextual node information is not available.
        /// </summary>
        /// <value>
        /// A URL string, not null, not empty.
        /// </value>
        string Url
        {
            get;
        }

        /// <summary>
        /// Determines whether the specified node matches this node matcher.
        /// </summary>
        /// <param name="node">A non-null node to test.</param>
        /// <exception cref="ArgumentNullException">If <paramref name="node"/> has a null URL.</exception>
        /// <returns><c>true</c> or <c>false</c></returns>
        bool Matches(BreadCrumbNode node);
    }
}
