/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER
 * Version: 2.0
 */

using NUnit.Framework;

namespace TopCoder.Web.UI.WebControl.BreadCrumb
{
    /// <summary>
    /// This class tests the breadcrumb control in direct interaction with IIS.
    /// </summary>
    [TestFixture]
    public class HttpTestBreadCrumbControl : HttpTest
    {
        /// <summary>
        /// Tests "silent" error handling.
        /// </summary>
        [Test]
        public void TestRenderSilently()
        {
            Browser.GetPage(@"http://localhost/silent.aspx");
            VerifyString(@"<span id=""bcc""></span>");
        }

        /// <summary>
        /// Tests "throwing" error handling.
        /// </summary>
        [Test]
        [ExpectedException(typeof(NUnit.Extensions.Asp.HttpClient.AspServerException))]
        public void TestRenderThrow()
        {
            Browser.GetPage(@"http://localhost/throw.aspx");
        }

        /// <summary>
        /// Tests "rendering" error handling. We don't look for the complete message, as some parts are localized
        /// depending on current UI culture.
        /// </summary>
        [Test]
        public void TestRenderRender()
        {
            Browser.GetPage(@"http://localhost/render.aspx");
            NUnit.Framework.Assert.IsTrue(Browser.CurrentPageText.IndexOf(
                @"$START$<span id=""bcc""><p>TopCoder.Web.UI.WebControl.BreadCrumb.BreadCrumbException: Object fa") !=
                -1, "The exception should be rendered.");
        }

        /// <summary>
        /// Tests "manual" data binding, with a list.
        /// </summary>
        [Test]
        public void TestDataBindList()
        {
            Browser.GetPage(@"http://localhost/DataBindList.aspx");
            VerifyString(@"<span id=""bcc""><a href=""url1"">title1</a><span>:</span>" +
                @"<a href=""url2"">title2</a></span>");
        }

        /// <summary>
        /// Tests "manual" data binding, with a table.
        /// </summary>
        [Test]
        public void TestDataBindTable()
        {
            Browser.GetPage(@"http://localhost/DataBindTable.aspx");
            VerifyString(@"<span id=""bcc""><a href=""url1"">title1</a><span>:</span>" +
                @"<a href=""url2"">title2</a></span>");
        }

        /// <summary>
        /// Tests "manual" data binding, with a data set (and a table).
        /// </summary>
        [Test]
        public void TestDataBindSet()
        {
            Browser.GetPage(@"http://localhost/DataBindSet.aspx");
            VerifyString(@"<span id=""bcc""><a href=""url1"">title1</a><span>:</span>" +
                @"<a href=""url2"">title2</a></span>");
        }

        /// <summary>
        /// Tests specifying a lot of arguments in the tag.
        /// </summary>
        [Test]
        public void TestLotsOfArgs()
        {
            Browser.GetPage(@"http://localhost/LotsOfArgs.aspx");
            VerifyString(@"<span id=""bcc""><a href=""url2"" class=""CNS"" onmouseover=""this.className='M'"" " +
                @"onmouseout=""this.className='CNS'"">title2</a><span class=""Style"">,</span><a href=""url1"" " +
                @"class=""NodeStyle"" onmouseover=""this.className='M'"" onmouseout=""this.className='NodeStyle'"">" +
                @"title1</a></span>");
        }
    }
}
