/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER
 * Version: 2.0
 */

using NUnit.Framework;
using System;
using System.Collections;
using System.Web.UI;
using TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery.NodeMatcher;

namespace TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery
{
    /// <summary>
    /// This class tests <see cref="DynamicPathDiscovery"/>.
    /// </summary>
    [TestFixture]
    public class TestDynamicPathDiscovery
    {
        /// <summary>
        /// Two session variables are used to cache the sitemap data. This test just verifies the constant values.
        /// </summary>
        [Test]
        public void TestConstants()
        {
            Assert.AreEqual("TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery.DynamicPathDiscovery.siteMap",
                DynamicPathDiscovery.CURRENT_SITEMAP);
            Assert.AreEqual("TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery.DynamicPathDiscovery.reversedSiteMap",
                DynamicPathDiscovery.REVERSED_SITEMAP);
            Assert.AreEqual("TopCoder.Web.UI.WebControl.BreadCrumb.PathDiscovery.DynamicPathDiscovery.currentPath",
                DynamicPathDiscovery.CURRENT_PATH);
        }

        /// <summary>
        /// Tests against a null node URL. An exception is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestNullNodeUrl()
        {
            new DynamicPathDiscovery().GetPath(new BreadCrumbNode(), new Page());
        }

        /// <summary>
        /// Tests against a null page. An exception is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestNullPage()
        {
            new DynamicPathDiscovery().GetPath(new BreadCrumbNode("title", "url"), null);
        }
    }
}
