/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER
 * Version: 2.0
 */

using NUnit.Framework;
using System;
using TopCoder.Util.ConfigurationManager;
using System.Collections;
using System.Data;

namespace TopCoder.Web.UI.WebControl.BreadCrumb
{
    /// <summary>
    /// This class tests the entry point of the component, <c>BreadCrumbTrailControl</c>.
    /// </summary>
    [TestFixture]
    public class TestBreadCrumbTrailControl
    {
        /// <summary>
        /// The object used in most tests.
        /// </summary>
        private BreadCrumbTrailControl testObject;

        /// <summary>
        /// Clear all configuration manager data, we want to load the default settings of the control.
        /// </summary>
        [SetUp]
        public void Setup()
        {
            // Don't preload anything
            ConfigManager.GetInstance().Clear(false);

            testObject = new BreadCrumbTrailControl();
        }

        /// <summary>
        /// Tests checking the default value, setting a new value, reading it back.
        /// </summary>
        [Test]
        public void TestPathSeparator()
        {
            Assert.AreEqual(":", testObject.PathSeparator, "Default value mismatch.");
            testObject.PathSeparator = "blah";

            Assert.AreEqual("blah", testObject.PathSeparator, "New value not correctly set.");
        }

        /// <summary>
        /// Tests setting a null value, which is allowed.
        /// </summary>
        [Test]
        public void TestPathSeparatorNull()
        {
            testObject.PathSeparator = null;
            Assert.IsNull(testObject.PathSeparator, "Set null <=> get null.");
        }

        /// <summary>
        /// Tests setting an empty string. This is allowed, and for path separators, the string is actually preserved.
        /// (Not translated to null.)
        /// </summary>
        [Test]
        public void TestPathSeparatorEmpty()
        {
            testObject.PathSeparator = " ";
            Assert.AreEqual(" ", testObject.PathSeparator, "Set empty <=> get empty.");

            // The "real" empty string
            testObject.PathSeparator = "";
            Assert.AreEqual("", testObject.PathSeparator, "Set empty <=> get empty.");
        }

        /// <summary>
        /// Tests checking the default value, setting a new value, reading it back.
        /// </summary>
        [Test]
        public void TestPathSeparatorStyle()
        {
            Assert.AreEqual(null, testObject.PathSeparatorStyle, "Default value mismatch.");
            testObject.PathSeparatorStyle = "blah";

            Assert.AreEqual("blah", testObject.PathSeparatorStyle, "New value not correctly set.");
        }

        /// <summary>
        /// Tests setting a null value, which is allowed.
        /// </summary>
        [Test]
        public void TestPathSeparatorStyleNull()
        {
            testObject.PathSeparatorStyle = null;
            Assert.IsNull(testObject.PathSeparatorStyle, "Set null <=> get null.");
        }

        /// <summary>
        /// Tests setting an empty string. This is allowed, but the value is translated into null.
        /// </summary>
        [Test]
        public void TestPathSeparatorStyleEmpty()
        {
            testObject.PathSeparatorStyle = " ";
            Assert.IsNull(testObject.PathSeparatorStyle, "Set empty <=> get null.");

            // The "real" empty string
            testObject.PathSeparatorStyle = "";
            Assert.IsNull(testObject.PathSeparatorStyle, "Set empty <=> get null.");
        }

        /// <summary>
        /// Tests checking the default value, setting a new value, reading it back.
        /// </summary>
        [Test]
        public void TestNodeStyle()
        {
            Assert.AreEqual(null, testObject.NodeStyle, "Default value mismatch.");
            testObject.NodeStyle = "blah";

            Assert.AreEqual("blah", testObject.NodeStyle, "New value not correctly set.");
        }

        /// <summary>
        /// Tests setting a null value, which is allowed.
        /// </summary>
        [Test]
        public void TestNodeStyleNull()
        {
            testObject.NodeStyle = null;
            Assert.IsNull(testObject.NodeStyle, "Set null <=> get null.");
        }

        /// <summary>
        /// Tests setting an empty string. This is allowed, but the value is translated into null.
        /// </summary>
        [Test]
        public void TestNodeStyleEmpty()
        {
            testObject.NodeStyle = " ";
            Assert.IsNull(testObject.NodeStyle, "Set empty <=> get null.");

            // The "real" empty string
            testObject.NodeStyle = "";
            Assert.IsNull(testObject.NodeStyle, "Set empty <=> get null.");
        }

        /// <summary>
        /// Tests checking the default value, setting a new value, reading it back.
        /// </summary>
        [Test]
        public void TestCurrentNodeStyle()
        {
            Assert.AreEqual(null, testObject.CurrentNodeStyle, "Default value mismatch.");
            testObject.CurrentNodeStyle = "blah";

            Assert.AreEqual("blah", testObject.CurrentNodeStyle, "New value not correctly set.");
        }

        /// <summary>
        /// Tests setting a null value, which is allowed.
        /// </summary>
        [Test]
        public void TestCurrentNodeStyleNull()
        {
            testObject.CurrentNodeStyle = null;
            Assert.IsNull(testObject.CurrentNodeStyle, "Set null <=> get null.");
        }

        /// <summary>
        /// Tests setting an empty string. This is allowed, but the value is translated into null.
        /// </summary>
        [Test]
        public void TestCurrentNodeStyleEmpty()
        {
            testObject.CurrentNodeStyle = " ";
            Assert.IsNull(testObject.CurrentNodeStyle, "Set empty <=> get null.");

            // The "real" empty string
            testObject.CurrentNodeStyle = "";
            Assert.IsNull(testObject.CurrentNodeStyle, "Set empty <=> get null.");
        }

        /// <summary>
        /// Tests checking the default value, setting a new value, reading it back.
        /// </summary>
        [Test]
        public void TestRootNodeStyle()
        {
            Assert.AreEqual(null, testObject.RootNodeStyle, "Default value mismatch.");
            testObject.RootNodeStyle = "blah";

            Assert.AreEqual("blah", testObject.RootNodeStyle, "New value not correctly set.");
        }

        /// <summary>
        /// Tests setting a null value, which is allowed.
        /// </summary>
        [Test]
        public void TestRootNodeStyleNull()
        {
            testObject.RootNodeStyle = null;
            Assert.IsNull(testObject.RootNodeStyle, "Set null <=> get null.");
        }

        /// <summary>
        /// Tests setting an empty string. This is allowed, but the value is translated into null.
        /// </summary>
        [Test]
        public void TestRootNodeStyleEmpty()
        {
            testObject.RootNodeStyle = " ";
            Assert.IsNull(testObject.RootNodeStyle, "Set empty <=> get null.");

            // The "real" empty string
            testObject.RootNodeStyle = "";
            Assert.IsNull(testObject.RootNodeStyle, "Set empty <=> get null.");
        }

        /// <summary>
        /// Tests checking the default value, setting a new value, reading it back.
        /// </summary>
        [Test]
        public void TestMouseOverStyle()
        {
            Assert.AreEqual(null, testObject.MouseOverStyle, "Default value mismatch.");
            testObject.MouseOverStyle = "blah";

            Assert.AreEqual("blah", testObject.MouseOverStyle, "New value not correctly set.");
        }

        /// <summary>
        /// Tests setting a null value, which is allowed.
        /// </summary>
        [Test]
        public void TestMouseOverStyleNull()
        {
            testObject.MouseOverStyle = null;
            Assert.IsNull(testObject.MouseOverStyle, "Set null <=> get null.");
        }

        /// <summary>
        /// Tests setting an empty string. This is allowed, but the value is translated into null.
        /// </summary>
        [Test]
        public void TestMouseOverStyleEmpty()
        {
            testObject.MouseOverStyle = " ";
            Assert.IsNull(testObject.MouseOverStyle, "Set empty <=> get null.");

            // The "real" empty string
            testObject.MouseOverStyle = "";
            Assert.IsNull(testObject.MouseOverStyle, "Set empty <=> get null.");
        }

        /// <summary>
        /// Tests the path direction property, default value and changing it.
        /// </summary>
        [Test]
        public void TestPathDirection()
        {
            Assert.AreEqual(PathDirection.RootToCurrent, testObject.PathDirection, "Default value mismatch.");

            testObject.PathDirection = PathDirection.CurrentToRoot;
            Assert.AreEqual(PathDirection.CurrentToRoot, testObject.PathDirection, "New value not correctly set.");

            testObject.PathDirection = PathDirection.RootToCurrent;
            Assert.AreEqual(PathDirection.RootToCurrent, testObject.PathDirection,
                "Old value not correctly restored.");
        }

        /// <summary>
        /// Tests setting a value which is not a valid <see cref="PathDirection"/>. This should result in an exception.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestPathDirectionInvalid()
        {
            testObject.PathDirection = (PathDirection) 2;
        }

        /// <summary>
        /// Tests the path direction property, default value and changing it.
        /// </summary>
        [Test]
        public void TestErrorDisposition()
        {
            Assert.AreEqual(ErrorDisposition.SilentlyIgnore, testObject.ErrorDisposition, "Default value mismatch.");

            testObject.ErrorDisposition = ErrorDisposition.RenderException;
            Assert.AreEqual(ErrorDisposition.RenderException, testObject.ErrorDisposition,
                "New value not correctly set.");

            testObject.ErrorDisposition = ErrorDisposition.ThrowException;
            Assert.AreEqual(ErrorDisposition.ThrowException, testObject.ErrorDisposition,
                "New value not correctly set.");

            testObject.ErrorDisposition = ErrorDisposition.SilentlyIgnore;
            Assert.AreEqual(ErrorDisposition.SilentlyIgnore, testObject.ErrorDisposition,
                "Old value not correctly restored.");
        }

        /// <summary>
        /// Tests setting a value which is not a valid <see cref="ErrorDisposition"/>. This should result in an
        /// exception.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestErrorDispositionInvalid()
        {
            testObject.ErrorDisposition = (ErrorDisposition) 3;
        }

        /// <summary>
        /// The data source is not allowed to be empty, if it's an <see cref="IList"/>. An exception is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestDataSourceIListEmpty()
        {
            testObject.DataSource = new ArrayList();
        }

        /// <summary>
        /// The data source is not allowed to contain non-<c>BreadCrumbNode</c> elements, if it's an
        /// <see cref="IList"/>. An exception is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestDataSourceIListNonNode()
        {
            // An object array is also an IList
            testObject.DataSource = new object[] { "Not a node" };
        }

        /// <summary>
        /// The data source is not allowed to contain null elements, if it's an <see cref="IList"/>. An exception is
        /// thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestDataSourceIListNullElement()
        {
            // An object array is also an IList
            testObject.DataSource = new object[] { null };
        }

        /// <summary>
        /// The data source is not allowed to contain <c>BreadCrumbNode</c> elements with a null URL (created by the
        /// default constructor), if it's an <see cref="IList"/>. An exception is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestDataSourceIListNullURLNode()
        {
            // An object array is also an IList
            testObject.DataSource = new object[] { new BreadCrumbNode() };
        }

        /// <summary>
        /// The data source is not allowed to be empty, if it's an array. An exception is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestDataSourceArrayEmpty()
        {
            testObject.DataSource = new BreadCrumbNode[0];
        }

        /// <summary>
        /// The data source is not allowed to contain <c>BreadCrumbNode</c> elements with a null URL (created by the
        /// default constructor), if it's an array. An exception is thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestDataSourceArrayNullURLNode()
        {
            testObject.DataSource = new BreadCrumbNode[1];
        }

        /// <summary>
        /// The data source, if it's a data table, should contain both a URL and a Title column. That's not the case
        /// here. An exception is thrown.
        /// </summary>
        [ExpectedException(typeof(ArgumentException))]
        [Test]
        public void TestDataSourceDataTableNoUrl()
        {
            DataTable table = new DataTable();
            table.Columns.Add("Title");

            testObject.DataSource = table;
        }

        /// <summary>
        /// The data source, if it's a data table, should contain both a URL and a Title column. That's not the case
        /// here. An exception is thrown.
        /// </summary>
        [ExpectedException(typeof(ArgumentException))]
        [Test]
        public void TestDataSourceDataTableNoTitle()
        {
            DataTable table = new DataTable();
            table.Columns.Add("Url");

            testObject.DataSource = table;
        }

        /// <summary>
        /// If the data source is a data set, it must contain a table. This test case has no tables. An exception is
        /// thrown.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestDataSourceDataSetEmpty()
        {
            DataSet set = new DataSet();

            testObject.DataSource = set;
        }

        /// <summary>
        /// The data source, if it's a data set, should contain both a URL and a Title column in the first table.
        /// That's not the case here. An exception is thrown.
        /// </summary>
        [ExpectedException(typeof(ArgumentException))]
        [Test]
        public void TestDataSourceDataSetNoUrl()
        {
            DataTable table = new DataTable();
            table.Columns.Add("Title");

            DataSet set = new DataSet();
            set.Tables.Add(table);
            testObject.DataSource = set;
        }

        /// <summary>
        /// The data source, if it's a data set, should contain both a URL and a Title column in the first table.
        /// That's not the case here. An exception is thrown.
        /// </summary>
        [ExpectedException(typeof(ArgumentException))]
        [Test]
        public void TestDataSourceDataSetNoTitle()
        {
            DataTable table = new DataTable();
            table.Columns.Add("Url");

            DataSet set = new DataSet();
            set.Tables.Add(table);
            testObject.DataSource = set;
        }


        /// <summary>
        /// Tests checking the default value, setting a new value, reading it back.
        /// </summary>
        [Test]
        public void TestTitle()
        {
            Assert.AreEqual(null, testObject.Title, "Default value mismatch.");
            testObject.Title = "blah";

            Assert.AreEqual("blah", testObject.Title, "New value not correctly set.");
        }

        /// <summary>
        /// Tests setting a null value, which is allowed.
        /// </summary>
        [Test]
        public void TestTitleNull()
        {
            testObject.Title = null;
            Assert.IsNull(testObject.Title, "Set null <=> get null.");
        }

        /// <summary>
        /// Tests setting an empty string. This is allowed, but the value is translated into null.
        /// </summary>
        [Test]
        public void TestTitleEmpty()
        {
            testObject.Title = " ";
            Assert.IsNull(testObject.Title, "Set empty <=> get null.");

            // The "real" empty string
            testObject.Title = "";
            Assert.IsNull(testObject.Title, "Set empty <=> get null.");
        }

        /// <summary>
        /// Tests checking the default value, setting a new value, reading it back.
        /// </summary>
        [Test]
        public void TestNodeDiscoveryKey()
        {
            Assert.AreEqual(null, testObject.NodeDiscoveryKey, "Default value mismatch.");
            testObject.NodeDiscoveryKey = "blah";

            Assert.AreEqual("blah", testObject.NodeDiscoveryKey, "New value not correctly set.");
        }

        /// <summary>
        /// Tests setting a null value, which is allowed.
        /// </summary>
        [Test]
        public void TestNodeDiscoveryKeyNull()
        {
            testObject.NodeDiscoveryKey = null;
            Assert.IsNull(testObject.NodeDiscoveryKey, "Set null <=> get null.");
        }

        /// <summary>
        /// Tests setting an empty string. This is allowed, but the value is translated into null.
        /// </summary>
        [Test]
        public void TestNodeDiscoveryKeyEmpty()
        {
            testObject.NodeDiscoveryKey = " ";
            Assert.IsNull(testObject.NodeDiscoveryKey, "Set empty <=> get null.");

            // The "real" empty string
            testObject.NodeDiscoveryKey = "";
            Assert.IsNull(testObject.NodeDiscoveryKey, "Set empty <=> get null.");
        }

        /// <summary>
        /// Tests checking the default value, setting a new value, reading it back.
        /// </summary>
        [Test]
        public void TestPathDiscoveryKey()
        {
            Assert.AreEqual(null, testObject.PathDiscoveryKey, "Default value mismatch.");
            testObject.PathDiscoveryKey = "blah";

            Assert.AreEqual("blah", testObject.PathDiscoveryKey, "New value not correctly set.");
        }

        /// <summary>
        /// Tests setting a null value, which is allowed.
        /// </summary>
        [Test]
        public void TestPathDiscoveryKeyNull()
        {
            testObject.PathDiscoveryKey = null;
            Assert.IsNull(testObject.PathDiscoveryKey, "Set null <=> get null.");
        }

        /// <summary>
        /// Tests setting an empty string. This is allowed, but the value is translated into null.
        /// </summary>
        [Test]
        public void TestPathDiscoveryKeyEmpty()
        {
            testObject.PathDiscoveryKey = " ";
            Assert.IsNull(testObject.PathDiscoveryKey, "Set empty <=> get null.");

            // The "real" empty string
            testObject.PathDiscoveryKey = "";
            Assert.IsNull(testObject.PathDiscoveryKey, "Set empty <=> get null.");
        }

        /// <summary>
        /// Tests checking the default value, setting a new value, reading it back.
        /// </summary>
        [Test]
        public void TestPathWriterKey()
        {
            Assert.AreEqual(null, testObject.PathWriterKey, "Default value mismatch.");
            testObject.PathWriterKey = "blah";

            Assert.AreEqual("blah", testObject.PathWriterKey, "New value not correctly set.");
        }

        /// <summary>
        /// Tests setting a null value, which is allowed.
        /// </summary>
        [Test]
        public void TestPathWriterKeyNull()
        {
            testObject.PathWriterKey = null;
            Assert.IsNull(testObject.PathWriterKey, "Set null <=> get null.");
        }

        /// <summary>
        /// Tests setting an empty string. This is allowed, but the value is translated into null.
        /// </summary>
        [Test]
        public void TestPathWriterKeyEmpty()
        {
            testObject.PathWriterKey = " ";
            Assert.IsNull(testObject.PathWriterKey, "Set empty <=> get null.");

            // The "real" empty string
            testObject.PathWriterKey = "";
            Assert.IsNull(testObject.PathWriterKey, "Set empty <=> get null.");
        }

        /// <summary>
        /// Loads the specified configuration and verifies the contents against the current test object.
        /// </summary>
        /// <param name="filename">The configuration file to load from.</param>
        private void LoadConfigCheckAgainst(string filename)
        {
            ConfigManager.GetInstance().LoadFile(filename);

            // The configuration is used in the constructor
            BreadCrumbTrailControl control = new BreadCrumbTrailControl();

            Assert.AreEqual(testObject.PathSeparator, control.PathSeparator, "PathSeparator");
            Assert.AreEqual(testObject.PathSeparatorStyle, control.PathSeparatorStyle, "PathSeparatorStyle");
            Assert.AreEqual(testObject.NodeStyle, control.NodeStyle, "NodeStyle");
            Assert.AreEqual(testObject.CurrentNodeStyle, control.CurrentNodeStyle, "CurrentNodeStyle");
            Assert.AreEqual(testObject.RootNodeStyle, control.RootNodeStyle, "RootNodeStyle");
            Assert.AreEqual(testObject.MouseOverStyle, control.MouseOverStyle, "MouseOverStyle");
            Assert.AreEqual(testObject.ErrorDisposition, control.ErrorDisposition, "ErrorDisposition");
            Assert.AreEqual(testObject.PathDirection, control.PathDirection, "PathDirection");
            Assert.AreEqual(testObject.NodeDiscoveryKey, control.NodeDiscoveryKey, "NodeDiscoveryKey");
            Assert.AreEqual(testObject.PathDiscoveryKey, control.PathDiscoveryKey, "PathDiscoveryKey");
            Assert.AreEqual(testObject.PathWriterKey, control.PathWriterKey, "PathWriterKey");

            // The data source is not loaded by config.
            Assert.AreEqual(testObject.Title, control.Title, "Title");
        }

        /// <summary>
        /// Tests a "valid" configuration, some exceptions are thrown, but they are suppressed. The different reading
        /// results are verified.
        /// </summary>
        [Test]
        public void TestValidConfig1()
        {
            testObject.PathSeparator = "";
            testObject.PathSeparatorStyle = "stylish";
            testObject.PathDirection = PathDirection.CurrentToRoot;

            LoadConfigCheckAgainst(@"..\..\test_files\valid.xml");
        }

        /// <summary>
        /// Tests a "valid" configuration, completely empty, but with the namespace. This could, theoretically mess
        /// up some of the default values. Check that this is not the case!
        /// </summary>
        [Test]
        public void TestValidConfig2()
        {
            LoadConfigCheckAgainst(@"..\..\test_files\valid2.xml");
        }

        /// <summary>
        /// Restore all preloaded configuration manager data, as other tests may rely on something there.
        /// </summary>
        [TestFixtureTearDown]
        public void TearDownFixture()
        {
            // Preload everything
            ConfigManager.GetInstance().Clear(true);
        }
    }
}
